#include <QtGui>
#include <cmath>
#include <cstdlib>

#include "mainwindow.h"
#include "functions.h"

MainWindow::MainWindow(QWidget *parent)
	: QMainWindow(parent, 0)
{
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(timerCode()));
	timer->start(100);
	ui.setupUi(this);
	
	pacMan = new Pac(width()/2, height()/2);
	balls.clear();
	anim = false;
	time = 0;
	FILE * fp = fopen(QFile::encodeName("time.qpac"), "r");
	if (fp != NULL) {
		fscanf(fp, "%X", &toptime);
		fclose(fp);
		toptime = int(sqrt(toptime / 1987));
		ui.toptime->setText(QString::number(toptime));
	} else {
        toptime = 0;
	}
	timerCount = 0;
	update();
}

void MainWindow::closeEvent(QCloseEvent* event) {
	Q_UNUSED(event)
	FILE * fp = fopen(QFile::encodeName("time.qpac"), "w");
	if (fp != NULL) {
		fprintf(fp, "%X", int(toptime * toptime * 1987));
		fclose(fp);
	} else {
        QMessageBox mb(windowTitle(),
					"Couldn't save top time!",
                           QMessageBox::Critical,
                           QMessageBox::Ok,
                           QMessageBox::NoButton,
                           QMessageBox::NoButton);
		mb.exec();
	}
}

void MainWindow::resizeEvent(QResizeEvent* event) {
	Q_UNUSED(event)
    //Calclate new number of balls, depending on the new area
	int newNum = int(ceil((height() * width()) / 30720.0));
	//To make balls more widely spread in the new area, rinse and repeat creation.
	balls.clear();
	for (int i = 0; i < newNum; i++) {
		balls << new Ball(BORDER+rand()%(width()-(BORDER*2)), BORDER+rand()%(height()-(BORDER*2)), rand()%361);
	}
 	ui.balls->setText(QString::number(balls.size()));
	//Remove off screen dots
	for (int i = 0; i < dots.size(); i++) {
		if ((dots.at(i).x() > width()) || (dots.at(i).y() > height())) {
			QPolygon::iterator it = dots.begin() + i;
			dots.erase(it);
		}
	}
 	//Move pacMan to inside the new screen (if he disappeared off screen)
    QPoint pacPos = pacMan->getPos();
    if (pacPos.x() > width()) {
		pacPos.setX(width());
	}
	if (pacPos.y() > height()) {
		pacPos.setY(height());
	}
	if (pacPos != pacMan->getPos()) {
		pacMan->setPos(pacPos);
	}
}

void MainWindow::paintEvent(QPaintEvent* event) {
	Q_UNUSED(event)
	QPainter p(this);
	p.setPen(QPen(Qt::green,9));
	p.drawPoints(dots);
	if (anim) {
		QPoint pacPos = pacMan->getPos();
		if ((dots.size() > 0) && (!pacMan->isDying())) {
			int p = getClosest(dots, pacPos);
			double dist = pointDist(dots.at(p), pacPos);
			if (dist > 20.0) { //don't move if too near dot, it seems the pacMan sometimes dissapear otherwise
				dist = pacMan->moveTowards(dots.at(p));
			}
			pacPos = pacMan->getPos();
			pacMan->chew();
			if (dist <= 20.0) {
				QPolygon::iterator it = dots.begin() + p;
				dots.erase(it);
				if (dots.size() < 1) {
					pacMan->chew(true);
				} else {
					p = getClosest(dots, pacPos);
					pacMan->turnTowards(dots.at(p));
				}
			}
		}
		for (int i = 0; i < balls.size(); i++) {
			balls.at(i)->moveAlong(width(),height());
			if (pacMan->isDying()) { //Do this afterwards because I don't want two loops...
				balls.at(i)->setSpeed(SSPEED);
			} else if (pointDist(balls.at(i)->getPos(), pacPos) <= 25.0) { //ball radius + pac radius = 25.
				ui.lasttime->setText(QString::number(time));
				time = 0;
				ui.time->setText(QString::number(0));
				timerCount = 0;
				pacMan->die(); //Kill the pacMan
				dots.clear(); //Remove uneaten dots
			}
		}
		anim = false;
	}
	for (int i = 0; i < balls.size(); i++) {
		balls.at(i)->draw(p);
	}
	pacMan->draw(p);
}		
	
void MainWindow::mousePressEvent(QMouseEvent* event) {
	if (!pacMan->isDying()) {
		dots << QPoint(event->x(), event->y());
		update();
	}
}

void MainWindow::timerCode() {
	anim = true;
	if (!pacMan->isDying()) {
		timerCount++;
		if (timerCount >= 10) {
			time++;
			if (time % 10 == 0) { //Every ten points, increase speed
				int newSpeed = SSPEED + time/5;
				pacMan->setSpeed(newSpeed);
				for (int i = 0; i < balls.size(); i++) {
					balls.at(i)->setSpeed(newSpeed);
				}
			}
			ui.time->setText(QString::number(time));
			if (time > toptime) {
				toptime = time;
				ui.toptime->setText(QString::number(toptime));
			}
			timerCount = 0;
		}
	}
	update();
}
	
