#include <cmath>

#include "pac.h"
#include "functions.h"

Pac::Pac(int _x, int _y) : Ball(_x, _y, 0) {
	big = false;
	dying = false;
	angStart = 20;
	angSpan = 330;
}

void Pac::turnTowards(QPoint point) {
	dir = int(- atan2(point.y() - y, point.x() - x) * DEG);
	if (dir < 0) {
		dir += 360;
	}
}

double Pac::moveTowards(QPoint point) {
	turnTowards(point);
	double dist = pointDist(point, getPos()), s = speed/dist;
	x = int((1-s)*x + s*point.x());
	y = int((1-s)*y + s*point.y());
	return pointDist(point, getPos());
}

void Pac::draw(QPainter& p) {
	if (!dying) {
		//Calculate stuff
		if (big) {
			angStart = int(40 + dir);
			angSpan = 290;
		} else {
			angStart = int(20 + dir);
			angSpan = 330;
		}
		int eyeAngle = ((dir >= 90) && (dir <= 270) ? angStart + angSpan - 30 : angStart + 30);
		int eyeX = int(x + 14 * cos(-eyeAngle / DEG)), eyeY = int(y + 14 * sin(-eyeAngle / DEG));
		//Draw PacMan
		p.setPen(QPen(Qt::yellow, 1));
		p.setBrush(Qt::yellow);
		p.drawPie(int(x - 20), int(y - 20), 40, 40, angStart * 16, angSpan * 16);
		//Draw eyeball
		p.setPen(QPen(Qt::black, 1));
		p.setBrush(Qt::white);
		p.drawEllipse(eyeX - 3, eyeY - 3, 6, 6);
		//Draw pupil
		p.setBrush(Qt::black);
		p.drawEllipse(eyeX, eyeY, 2, 2);
	} else {
		//Animate death
		angSpan -= 15;
		angStart += 45;
		p.setPen(QPen(Qt::darkYellow, 1));
		p.setBrush(Qt::darkYellow);
		p.drawPie(int(x - 20), int(y - 20), 40, 40, angStart * 16, angSpan * 16);
		p.setPen(Qt::cyan);
		p.setFont(QFont("Arial",8));
		p.drawText(QRect(x - 30, y - 15, 60, 30), Qt::AlignCenter, "Game Over!\n(QPac died)");
		if (angSpan <= 0) { //animation end
			dir = 0;
			big = false;
			dying = false;
			x = p.device()->width() / 2;
			y = p.device()->height() / 2;
		}
	}
}

void Pac::chew(bool close) {
	big = !(big || close); //Logic! (Always false if close is true, otherwise toggles the value)
}

void Pac::die() {
	if (!dying) {
		speed = SSPEED;
		angSpan = 330;
		dying = true;
	}
}

bool Pac::isDying() {
	return dying;
}
